<?php



/* ------------------------------------------------- Form building */



/**
 * Builds preference form elements
 */
function _build_preferences($theme_prefix, $theme_settings, $options, &$form){
  
  $form['preferences_fieldset'] = array(
    '#title' => t('Preferences'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE, 
    '#collapsed' => TRUE, 
  );  
  
  $form['preferences_fieldset'][$theme_prefix.'stylesheet'] = array(
    '#title' => t('Generate stylesheet'),
    '#type' => 'checkbox',
    '#default_value' => $theme_settings[$theme_prefix.'stylesheet'],
    '#description' => 
      t('Generate a stylesheet for CSS generated by theme settings. By default
        styles are included in the page header to facilitate site development.
        Once theme settings have been finalised, turn this option on.<br/>NOTE:
        when this option is enabled, changes to the theme settings will only be
        visible upon browser cache refresh.'),
  );
  
  $form['preferences_fieldset'][$theme_prefix.'firebug'] = array(
    '#title' => t('Enable Firebug lite'),
    '#type' => 'checkbox',
    '#default_value' => $theme_settings[$theme_prefix.'firebug'],
    '#description' => 
      t('Enabling this option embeds the <em>Lite</em> version of
        <a href="http://getfirebug.com">Firebug</a> into pages. This option is
        purely for development, and should be disabled on live sites.'),
  );
  
  $form['preferences_fieldset'][$theme_prefix.'margin_increment'] = array(
    '#type' => 'select',
    '#title' => t('Margin grid size'),
    '#options' => $options['increment'],
    '#default_value' => $theme_settings[$theme_prefix.'margin_increment'],
    '#description' => t('Size of increments for selectable margins'),
  );
  
  $form['preferences_fieldset'][$theme_prefix.'padding_increment'] = array(
    '#title' => t('Padding grid size'),
    '#type' => 'select',
    '#options' => $options['increment'],
    '#default_value' => $theme_settings[$theme_prefix.'padding_increment'],
    '#description' => t('Size of increments for selectable padding'),
  );
  
}



/**
 * Builds the settings persistence
 */
function _build_persistence($theme_prefix, $theme_settings, &$form){
  
  $importable = file_exists(realpath(file_directory_path().'/fluid.settings'));
  
  $form['persistence_fieldset'] = array(
    '#title' => t('Import / Export settings'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE, 
    '#collapsed' => !$importable, 
  );
  
  $form['persistence_fieldset'][$theme_prefix.'export_settings'] = array(
    '#title' => t('Export settings'),
    '#type' => 'checkbox',
    '#default_value' => '0',
    '#description' => 
      t('Select this option to export the current Fluid settings to the files 
        folder'),
  );
  
  if ($importable){
    $form['persistence_fieldset'][$theme_prefix.'import_settings'] = array(
      '#title' => t('Import settings'),
      '#type' => 'checkbox',
      '#default_value' => '0',
      '#description' => 
        t('Select this option to import the Fluid settings present in the files
          folder'),
    );
  } else {
    $form['persistence_fieldset'][$theme_prefix.'import_settings'] = array(
      '#prefix' => '<div>',
      '#value' => 
        t('To import theme settings, place the fluid.settings file in 
          the root of the site files folder'),
      '#suffix' => '</div>',
    );
  }

}



/**
 * Builds shared form elements
 */
function _build_shared($theme_prefix, $theme_settings, $options, &$form){
    
  foreach (_fluid_custom_elements() as $element_key=>$element_info){
    
    $element_key = ereg_replace('[^A-Za-z0-9\_]', '_', $element_key);

    $plural = strtolower(substr(
        $element_info['section title'], 
        strlen($element_info['section title']
      ) - 1)) == 's';
    
    $form[$element_key.'_fieldset'] = array(
      '#type' => 'fieldset',
      '#title' => t($element_info['section title']), 
      '#collapsible' => TRUE, 
      '#collapsed' => TRUE, 
    );
    
    if ($element_info['title']){
      _build_title($theme_prefix, $theme_settings, $element_key, $form);
    }
    if ($element_info['border']){
      _build_border(
        $theme_prefix, $theme_settings, $plural, $element_key, 
        $element_info, $form
      );
    }
    if ($element_info['color']){
      _build_color(
        $theme_prefix, $theme_settings, $element_key, $element_info, $form
      );
    }
    if ($element_info['margin']){
      _build_margin(
        $theme_prefix, $theme_settings, $options, $plural, $element_key, 
        $element_info, $form
      );
    }
    if ($element_info['padding']){
      _build_padding(
        $theme_prefix, $theme_settings, $options, $plural, $element_key, 
        $element_info, $form
      );
    }
  }
  
}



/**
 * Builds title form element
 */
function _build_title($theme_prefix, $theme_settings, $element_key, &$form){
  
  $form[$element_key.'_fieldset'][$theme_prefix.$element_key.'_title_inside'] = 
    array(
      '#type' => 'checkbox',
      '#title' => t('Title inside border'),
      '#default_value' => 
        $theme_settings[$theme_prefix.$element_key.'_title_inside'],
    );
  
}



/**
 * Builds border form elements
 */
function _build_border(
          $theme_prefix, $theme_settings, $plural, $element_key, 
          $element_info, &$form){
  
  $form[$element_key.'_fieldset'][$element_key.'_border'] = array(
    '#type' => 'fieldset',
    '#title' => t('Border elements'), 
    '#collapsible' => TRUE, 
    '#collapsed' => TRUE, 
  );
  
  $form[$element_key.'_fieldset']
       [$element_key.'_border']
       [$theme_prefix.$element_key.'_border'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Border elements'),
    '#options' => _fluid_border_options(),
    '#default_value' => $theme_settings[$theme_prefix.$element_key.'_border'],
    '#description' => 
      'Use sparingly. Each checked border element causes a &lt;div&gt; to be 
      wrapped around '.($plural?'':'the ').
      strtolower($element_info['section title']),
  );
  
}



/**
 * Builds margin form elements
 */
function _build_margin(
          $theme_prefix, $theme_settings, $options, $plural, $element_key, 
          $element_info, &$form){
  
  $form[$element_key.'_fieldset'][$theme_prefix.$element_key.'_margin'] = array(
    '#type' => 'select',
    '#title' => t('Margin'),
    '#options' => $options['margin'],
    '#default_value' => $theme_settings[$theme_prefix.$element_key.'_margin'],
    '#attributes' => _fluid_has_custom_field(),
    '#description' => 
      t('Margin applied to '.($plural?'':'the ').
      strtolower($element_info['section title'])),
  );
  
  $form[$element_key.'_fieldset'][$theme_prefix.$element_key.'_margin_custom'] =
    array(
    '#type' => 'textfield',
    '#default_value' => 
      $theme_settings[$theme_prefix.$element_key.'_margin_custom'],
    '#attributes' => 
      $theme_settings[$theme_prefix.$element_key.'_margin'] == 'custom'
        ? _fluid_custom_field()
        : _fluid_custom_field_disabled(),
    '#description' => 
      'In the form: TTpx [RRpx [BBpx [LLpx]]] 
      eg. "5px", "10px 2px 1px" or "1px 3px"',
    '#validate' => 
      array('_validate_custom_margin' => array($element_key.' margin')),
  );

}



/**
 * Builds padding form elements
 */
function _build_padding(
          $theme_prefix, $theme_settings, $options, $plural, $element_key, 
          $element_info, &$form){

  $form[$element_key.'_fieldset'][$theme_prefix.$element_key.'_padding'] = 
  array(
    '#title' => t('Padding'),
    '#type' => 'select',
    '#options' => $options['padding'],
    '#default_value' => $theme_settings[$theme_prefix.$element_key.'_padding'],
    '#attributes' => _fluid_has_custom_field(),
    '#description' => 
      'Padding applied to the <strong>innermost border elememt</strong> of '.
      ($plural?'':'the ').strtolower($element_info['section title']).
      ' or to the <strong>'.strtolower($element_info['section title']).
      ($plural?' themselves':' itself').
      '</strong> if no border elements are rendered',
  );
  
  $form[$element_key.'_fieldset']
       [$theme_prefix.$element_key.'_padding_custom'] = array(
    '#type' => 'textfield',
    '#default_value' => 
      $theme_settings[$theme_prefix.$element_key.'_padding_custom'],
    '#attributes' => 
      $theme_settings[$theme_prefix.$element_key.'_padding'] == 'custom'
        ? _fluid_custom_field()
        : _fluid_custom_field_disabled(),
    '#description' => t('In the form: TTpx [RRpx [BBpx [LLpx]]]'),
    '#validate' => 
      array('_validate_custom_padding' => array($element_key.' padding')),
  );
  
}



/**
 * Builds color form elements
 */
function _build_color(
          $theme_prefix, $theme_settings, $element_key, 
          $element_info, &$form){

  $form[$element_key.'_fieldset'][$theme_prefix.$element_key.'_color'] = array(
    '#type' => 'fieldset',
    '#title' => t('Color'.(count($element_info['color']) > 1 ? 's' : '')), 
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form[$element_key.'_fieldset']
       [$theme_prefix.$element_key.'_color'][] = array(
    '#value' => 
      '<div class="color-picker" id="'.
      $element_key.'-color-picker"></div>', 
  );
  
  foreach($element_info['color'] as $element=>$info){
    $form[$element_key.'_fieldset']
         [$theme_prefix.$element_key.'_color']
         [$theme_prefix.$element_key.'_use_'.$element.'_color'] = array(
      '#title' => t($info['title']),
      '#type' => 'checkbox',
      '#attributes' => array('class' => 'use-color'),
      '#default_value' => 
        $theme_settings[$theme_prefix.$element_key.'_use_'.$element.'_color'],
    );
    
    $form[$element_key.'_fieldset']
         [$theme_prefix.$element_key.'_color']
         [$theme_prefix.$element_key.'_'.$element.'_color'] = array(
      '#type' => 'textfield',
      '#attributes' => array('class' => 'has-color-picker'),
      '#default_value' => 
        $theme_settings[$theme_prefix.$element_key.'_'.$element.'_color'],
    );
  }
}



/**
 * Builds extra page form elements
 */
function _build_page($theme_prefix, $theme_settings, &$form){
  
  $form['page_fieldset'][$theme_prefix.'page_width'] = array(
    '#title' => t('Page width'),
    '#type' => 'select',
    '#options' => _fluid_page_width_options(),
    '#default_value' => $theme_settings[$theme_prefix.'page_width'],
    '#attributes' => _fluid_has_custom_field(),
  );
  
  $form['page_fieldset'][$theme_prefix.'page_width_custom'] = array(
    '#type' => 'textfield',
    '#default_value' => $theme_settings[$theme_prefix.'page_width_custom'],
    '#attributes' => 
      $theme_settings[$theme_prefix.'page_width'] == 'custom'  
        ? _fluid_custom_field()
        : _fluid_custom_field_disabled(), 
  );
  
}



/**
 * Builds extra column form elements
 */
function _build_column($theme_prefix, $theme_settings, &$form){
  
  $form['column_fieldset'][$theme_prefix.'column_width'] = array(
    '#title' => t('Column width'),
    '#type' => 'select',
    '#options' => _fluid_column_width_options(),
    '#default_value' => $theme_settings[$theme_prefix.'column_width'],
    '#attributes' => _fluid_has_custom_field(),
  );
  
  $form['column_fieldset'][$theme_prefix.'column_width_custom'] = array(
    '#type' => 'textfield',
    '#default_value' => $theme_settings[$theme_prefix.'column_width_custom'],
    '#attributes' => 
      $theme_settings[$theme_prefix.'column_width'] == 'custom'  
        ? _fluid_custom_field()
        : _fluid_custom_field_disabled(), 
  );
  
  $form['column_fieldset'][$theme_prefix.'column_position'] = array(
    '#title' => t('Columns positioned on'),
    '#type' => 'select',
    '#options' => _fluid_column_position_options(),
    '#default_value' => $theme_settings[$theme_prefix.'column_position'],
  );
  
  $form['column_fieldset'][$theme_prefix.'column_swop'] = array(
    '#title' => t('Swop columns'),
    '#type' => 'checkbox',
    '#default_value' => $theme_settings[$theme_prefix.'column_swop'],
  );
  
}



/* ---------------------------------------------------------- Option building */



/**
 * Builds numeric options for padding and margin
 */
function _build_numeric_options($theme_prefix, $theme_settings){
  
  $options = array();
  
  $options['increment'] = array();
  for ($index = 1; $index<=50; $index++){
    $options['increment'][$index] = $index;
  }
    
  $options['margin'] = array();
  $options['padding'] = array();
  for ($index = -5; $index<=5; $index++){
    if ($index>=0){
      $options['padding']
              [$index * $theme_settings[$theme_prefix.'padding_increment'].'px'] 
      = $index * $theme_settings[$theme_prefix.'padding_increment'].'px';
    }
    $options['margin']
            [$index * $theme_settings[$theme_prefix.'margin_increment'].'px'] 
      = $index * $theme_settings[$theme_prefix.'margin_increment'].'px';
  }
  $options['margin']['custom'] = 'custom';
  $options['padding']['custom'] = 'custom';
  
  return $options;
}



/* ---------------------------------------------------------- Form validation */



/**
 * Validates a custom margin field
 */
function _validate_custom_margin($element){
  
  $pattern = '^(-?[0-9]{1,5}(px)?\ ?){1,4}$';
  if (!ereg($pattern, $element['#value'])) {
    form_error($element, t('Invalid margin supplied. '));
  }
  
}



/**
 * Validates a custom padding field
 */
function _validate_custom_padding($element){
  
  $pattern = '^([0-9]{1,5}(px)?\ ?){1,4}$';
  if (!ereg($pattern, $element['#value'])) {
    form_error($element, t('Invalid padding supplied. '));
  }
  
}